from . import authentication, download_my_uploads, new_description, update_description
__all__ = ["authentication", "download_my_uploads",
           "new_description", "update_description"]
from . import _version
__version__ = _version.get_versions()['version']



"""Run via command line to update your like to dislike ratio in YouTube
comment descriptions."""
import argparse
import pandas as pd
from ytad.download_my_uploads import DownloadUploadsData
from ytad.authentication import Authenticate
from ytad.new_description import NewDescription
from ytad.update_description import UpdateDescription
import glob
import os
import boto3
from datetime import datetime
from pytz import timezone, utc


def upload_aws_bucket(
    file_source_path: str = "data/video_statistics.csv",
    aws_bucket_name: str = "youtubedescription",
) -> None:
    """
    Sending description files to bucket for storage.
    Parameters:
    ----------
    file_source_path: str
        Address of file you want to save.
    Return:
    ----------
    None
    """
    s3 = boto3.resource("s3")
    date_format = "mdy_%m_%d_%Y_hms_%H_%M_%S_%Z"
    date = datetime.now(tz=utc)
    date = date.astimezone(timezone("US/Eastern"))
    estDateTime = date.strftime(date_format)
    s3.meta.client.upload_file(
        file_source_path,  # file to send to s3.
        aws_bucket_name,  # bucketname
        "video_statistics_{}.csv".format(estDateTime),
    )  # new name of file.


def update(args: argparse.ArgumentParser) -> None:
    """Update YouTube description with most up to date like to dislikes using YouTube's Data API v3."""
    channel_id = args.id
    print("Verifying your API keys...")
    auth = Authenticate()
    youtube_id = auth.check_token_web_app_data_api()
    dmu = DownloadUploadsData(youtube=youtube_id)

    print("Repull data from YouTube?", args.update_df, type(args.update_df))
    if args.update_df.lower() == "yes":
        uploads_playlist_id = dmu.get_my_uploads_playlist_id()
        file_prefix = "data/my_videos_page_"
        if not os.path.exists('data'):
            os.makedirs('data')
        if uploads_playlist_id is not None:
            dmu.download_playlist_video_snippets(
                playlist_id=uploads_playlist_id, file_prefix=file_prefix, auth=auth
            )
        else:
            print("There is no uploaded videos playlist for this user.")
        # ------------------------------------------------------------------------------------------
        # Obtaining the likes and dislikes from YouTube API
        files = glob.glob("data/my_videos_page_*.json")
        videos = dmu.get_videos_from_json_files(files=files)
        print(f"Number of records; loaded {len(videos)}")

        if uploads_playlist_id is not None:
            dmu.download_like_dislike(
                channel_id=channel_id, auth=auth
            )
        else:
            print("There is no uploaded videos playlist for this user.")
    else:
        files = glob.glob("data/my_videos_page_*.json")
        videos = dmu.get_videos_from_json_files(files=files)

    # ------------------------------------------------------------------------------------------
    # Update Description for each YouTube video.
    video_stat = pd.read_csv("data/video_statistics.csv")
    print("Loading video statistics")
    print(video_stat)

    def updated_description(old_description: str, record: pd.DataFrame) -> str:
        ud = NewDescription()
        return ud.merge_metadata_description(old_description, record, args.tz)

    new_descriptions = {
        v["snippet"]["resourceId"]["videoId"]: updated_description(
            v["snippet"]["description"],
            video_stat.loc[
                video_stat["Video_IDs"] == v["snippet"]["resourceId"]["videoId"]
            ],
        )
        for v in videos
    }

    if input("Start update process (must type YES)? ") != "YES":
        return
    upd = UpdateDescription(youtube=youtube_id)
    for video_id, new_description in new_descriptions.items():
        upd.update_description_on_youtube(
            video_id=video_id,
            new_description=new_description,
            verify=args.verify_each_update,
        )
        print(
            f"Video ID: {video_id} has been updated -- Check out updated description."
        )
    if args.aws.lower() == "yes":
        print(f"Saving files to s3: {args.bucket}")
        upload_aws_bucket(aws_bucket_name=args.bucket)


# if __name__ == "__main__":
def main():
    """Pass in arguments to command line to execute update command.
    Example: python update_notifications.py --update_df=No --verify_each_update=yes"""
    parser = argparse.ArgumentParser(
        description="Inputs arguments to update YouTube Description with like to dislike ratio."
    )
    parser.add_argument(
        "--id",
        type=str,
        required=True,
        help="Your YouTube ID. Remember, you can only get your own dislikes.",
    )
    parser.add_argument(
        "--update_df",
        type=str,
        default="yes",
        required=False,
        help="If you already have most up to date dataframs in /data/, you can conserve number of requests by opting out. Yes/No",
    )
    parser.add_argument(
        "--verify_each_update",
        type=str,
        default="yes",
        required=False,
        help="Update YouTube video descriptions without verifying content? Yes/No",
    )
    parser.add_argument(
        "--aws",
        type=str,
        default="no",
        required=False,
        help="Set --aws=yes if you want to run this on an instance. This sends data file to S3 bucket.",
    )
    parser.add_argument(
        "--bucket",
        type=str,
        default="youtubedescription",
        required=False,
        help="If --aws=yes, then this argument will be the bucket name. --bucket=youtubedescription",
    )
    parser.add_argument(
        "--tz",
        type=str,
        default="est",
        required=False,
        help="Supports est or pst timezones.",
    )
    args = parser.parse_args()
    update(args)



#!/bin/env python
"""Original Code: https://github.com/mCodingLLC/mCodingYouTube Functions to
update a single video description on YouTube."""

import difflib
import sys


class UpdateDescription:
    def __init__(self, youtube):
        self.youtube = youtube
        return

    def confirm_diff(self, old: str, new: str) -> bool:
        print("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< OLD <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<")
        print(old)
        print(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> NEW  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<")
        print(new)
        print("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< DIFF <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<")
        differ = difflib.unified_diff(
            old.splitlines(keepends=True), new.splitlines(keepends=True)
        )
        sys.stdout.writelines(differ)
        print(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> DONE")
        while True:
            decision = input("Does this look correct [Y/n]? ")
            decision = decision.lower()
            if decision == "" or "y" in decision:
                return True
            elif "n" in decision:
                return False
            print("Invalid input, try again")

    def update_description_on_youtube(self, video_id, new_description, verify) -> None:
        if "<" in new_description or ">" in new_description:
            raise ValueError("new_description cannot contain < or >")

        # see https://developers.google.com/youtube/v3/docs/videos/list
        videos_list_response = (
            self.youtube.videos().list(id=video_id, part="snippet").execute()
        )

        if not videos_list_response["items"]:
            raise KeyError(f"Video {video_id} was not found.")

        # Since the request specified a video ID, the response only contains one
        # video resource. This code extracts the snippet from that resource.
        videos_list_snippet = videos_list_response["items"][0]["snippet"]

        if videos_list_snippet["description"] == new_description:
            print(
                f"Video {video_id}: new description and old description are the same, skipping..."
            )
            return

        # comment this if statement if you are 100% comforatble with what you will be changing in the description.
        if (
            verify.lower() == "yes"
        ):  # if false, verify. Else, it will update all parameters.
            if not self.confirm_diff(
                old=videos_list_snippet["description"], new=new_description
            ):
                print("diff rejected, aborting program...")
                sys.exit(0)

        # user has confirmed the new version looks good, go ahead
        videos_list_snippet["description"] = new_description

        # it seems like a bug in the youtube api that this needs to be done
        # a downloaded video may have no tags,
        # but omitting tags during upload results in 400 error
        if "tags" not in videos_list_snippet:
            videos_list_snippet["tags"] = []

        # see https://developers.google.com/youtube/v3/docs/videos/update
        videos_update_response = (
            self.youtube.videos()
            .update(part="snippet", body=dict(snippet=videos_list_snippet, id=video_id))
            .execute()
        )

        if videos_update_response["snippet"]["description"] != new_description:
            raise RuntimeError("update failed")



#!/bin/env python
"""Original Code: https://github.com/mCodingLLC/mCodingYouTube Get Metadata
from ALL of the Video uploads on YouTube.

Important Pieces of Metadata to extract:     videoId     description
"""
import json
from typing import List
import pandas as pd
from time import gmtime, strftime


class DownloadUploadsData:
    def __init__(self, youtube):
        self.youtube = youtube
        return

    def _dump_json_to_file(self, obj, filename: str) -> None:
        """Write data to json file."""
        with open(filename, "w", encoding="utf-8") as f:
            json.dump(obj, f)

    def get_my_uploads_playlist_id(self) -> str:
        """Get the list of playlist ids from each youtube video."""
        # see https://developers.google.com/youtube/v3/docs/channels/list
        channels_response = (
            self.youtube.channels().list(mine=True, part="contentDetails").execute()
        )

        for channel in channels_response["items"]:
            return channel["contentDetails"]["relatedPlaylists"]["uploads"]

        return None

    def download_playlist_video_snippets(self, playlist_id, file_prefix, auth) -> None:
        """Get playlist data from YouTube Channel.
        Parameters:
        ----------
        playlist_id: str
            Playlist id for each YouTube Video.
        file_prefix: str
            Starting filename for data referenced files in /data/
        auth:
            YouTube Credentials for API reference for your YouTube account.
        Return:
        ----------"""
        # see https://developers.google.com/youtube/v3/docs/playlistItems/list
        playlistitems_list_request = self.youtube.playlistItems().list(
            playlistId=playlist_id, part="snippet", maxResults=50
        )

        results = auth.paginated_results(
            self.youtube.playlistItems(), playlistitems_list_request
        )
        for page_no, playlistitems_list_response in enumerate(results):
            self._dump_json_to_file(
                playlistitems_list_response, f"{file_prefix}{page_no}.json"
            )

    def get_videos_from_json_files(self, files: List[str]) -> List[str]:
        """Gets videos from list of files.
        Parameters:
        ----------
        files: List[str]
            List of filenames that contain data references.
        Return:
        ----------
        List[str]:
            List of items related to videos.
        """
        videos = []
        for filename in files:
            with open(filename, "r", encoding="utf-8") as f:
                playlist_list_response = json.load(f)
                videos.extend(playlist_list_response["items"])
        return videos

    def download_like_dislike(self, channel_id, auth) -> None:
        """Get the current likes and dislikes of each YouTube Video.
        Parameters:
        ---------
        channel_id: str
            Your YouTube Channel ID.
        auth:
            YouTube Credentials for API reference for your YouTube account.
        Return:
        ---------
        None
        """
        requestVidId = self.youtube.search().list(
            part="snippet", channelId=channel_id, order="date", maxResults=50
        )
        results = auth.paginated_results(self.youtube.search(), requestVidId)
        video_names = []
        video_ids = []
        likes = []
        dislikes = []
        ratios = []
        date_stores = []
        c = 1
        for page_no, requestVidId_response in enumerate(results):
            for item in requestVidId_response["items"]:
                try:
                    vidId = item["id"]["videoId"]
                    title = item["snippet"]["title"]
                    # Get statistics
                    requestStats = self.youtube.videos().list(
                        part="statistics", id=vidId
                    )
                    responseStats = requestStats.execute()

                    # Like Dislike.
                    for item in responseStats["items"]:
                        like = item["statistics"]["likeCount"]
                        dislike = item["statistics"]["dislikeCount"]

                    ratio = float(like) / (float(like) + float(dislike)) * 100
                    currentDate = strftime("%Y-%m-%d %H:%M:%S", gmtime())
                    video_names.append(title)
                    video_ids.append(vidId)
                    likes.append(like)
                    dislikes.append(dislike)
                    ratios.append(ratio)
                    date_stores.append(currentDate)
                    print(
                        c,
                        "Video Name",
                        title,
                        "Video ID ",
                        item["id"],
                        like,
                        dislike,
                        ratio,
                        "Date Updated:",
                        currentDate,
                        "GMT",
                    )
                    c += 1
                except Exception:
                    print("Ignoring playlist pull..")
                    continue
        video_names = pd.DataFrame(video_names)
        video_ids = pd.DataFrame(video_ids)
        likes = pd.DataFrame(likes)
        dislikes = pd.DataFrame(dislikes)
        ratios = pd.DataFrame(ratios)
        date_stores = pd.DataFrame(date_stores)
        df = pd.concat(
            [video_names, video_ids, likes, dislikes, ratios, date_stores], axis=1
        )
        df.columns = ["Video_Names", "Video_IDs", "Likes", "Dislikes", "Ratios", "Date"]
        df.to_csv("data/video_statistics.csv", mode="w+", index=False)
        return None



#!/bin/env python
"""Original Code:
https://github.com/mCodingLLC/mCodingYouTube/tree/master/mcoding_youtube.

This is where you can edit what description your videos will be. For now, I will just create
the ability to update the like and dislike counter.

There is already a lot of good code here, so I will leave most of it here.
"""
from datetime import datetime
from pytz import timezone, utc
import pandas as pd


class NewDescription:
    def __init__(self):
        pass

    def _get_time(self, time_zone: str) -> None:
        """Get current time; supports est/pst.
        time_zone: str
            est or pst
        Parameters:
        ----------
        time_zone: str
            type of time zone standard you want to include for your timestamp.
        Return:
        ----------
        None"""
        date_format = "%m-%d-%Y %H:%M:%S %Z"
        date = datetime.now(tz=utc)
        if time_zone.lower() == "pst":
            date = date.astimezone(timezone("US/Pacific"))
            pstDateTime = date.strftime(date_format)
            return pstDateTime
        elif time_zone.lower() == "est":
            date = date.astimezone(timezone("US/Eastern"))
            estDateTime = date.strftime(date_format)
            return estDateTime
        return None

    def merge_metadata_description(self, old_description: str, record: pd.DataFrame, time_zone: str) -> str:
        """Update new description with the number of likes, dislikes, ratios, and time stamps.
        Parameters:
        ----------
        old_description: str
            old description of your YouTube Videos.
        record: pd.DataFrame
            the dataframe that holds metadata for each of your videos.
        timezone: str
            the timezone to be plugged in for timestamp. Supports EST/PST.
        Return:
        ----------
        str:
            new description to be updated.
        """
        current_dt = self._get_time(time_zone)  # Pst
        likes = record["Likes"].values[0]
        dislikes = record["Dislikes"].values[0]
        if likes == 0 and dislikes == 0:
            return old_description
        else:
            edit_line = f"===== Likes: {likes} 👍: Dislikes: {dislikes} 👎: {round(int(likes)/(int(likes) + int(dislikes)) * 100, 3)}% : Updated on {current_dt} ====="
            if old_description.startswith("====="):
                # Update the counter.
                edit_desc = old_description.splitlines()
                edit_desc[0] = edit_line
                edit_desc = "\n".join(edit_desc)
                return edit_desc
            else:
                # insert counter line.
                edit_line = edit_line + "\n" + old_description
                return edit_line



#!/bin/env python
"""Make sure to check out
https://github.com/mCodingLLC/mCodingYouTube/tree/master/mcoding_youtube Lots
of setup inspiration came from his code!

Wrapper around Google's OAuth2 authentication process providing
convenience functions to build service objects.
"""

from typing import Iterator
from google_auth_oauthlib.flow import InstalledAppFlow
from googleapiclient.discovery import build
import os
import pickle

# Google's Request
from google.auth.transport.requests import Request


class Authenticate:
    def __init__(self):
        pass

    def check_token_web_app_data_api(self):
        """Used for client_secret_web_app (Webapp keys from Google OAUTH)
        Source: Corey Schafer
            YouTube Video Link: https://www.youtube.com/watch?v=th5_9woFJmk&t=3s
        """
        credentials = None
        # token.pickle stores the user's credentials from previously successful logins
        if os.path.exists("token.pickle"):
            print("Loading Credentials From File...")
            with open("token.pickle", "rb") as token:
                credentials = pickle.load(token)
        # If there are no valid credentials available, then either refresh the token or log in.
        if not credentials or not credentials.valid:
            if credentials and credentials.expired and credentials.refresh_token:
                print("Refreshing Access Token...")
                credentials.refresh(Request())
            else:
                print("Fetching New Tokens...")
                flow = InstalledAppFlow.from_client_secrets_file(
                    "client_secret_web_app.json",
                    scopes=[
                        "https://www.googleapis.com/auth/youtube",
                        "https://www.googleapis.com/auth/youtube.readonly",
                        "https://www.googleapis.com/auth/youtube.force-ssl",
                    ],
                )

                flow.run_local_server(
                    port=8080, prompt="consent", authorization_prompt_message=""
                )
                credentials = flow.credentials

                # Save the credentials for the next run
                with open("token.pickle", "wb") as f:
                    print("Saving Credentials for Future Use...")
                    pickle.dump(credentials, f)
        youtube = build("youtube", "v3", credentials=credentials)
        return youtube

    def paginated_results(
        self, youtube_listable_resource, list_request, limit_requests=10
    ) -> Iterator:
        """Used to paginate results if data overflow; this iterates through web pages
        when there are more results on other pages to return."""
        remaining = -1 if limit_requests is None else limit_requests
        while list_request and remaining != 0:
            list_response = list_request.execute()
            yield list_response
            # see googleapiclient/discovery.py createNextMethod for *_next methods
            list_request = youtube_listable_resource.list_next(
                list_request, list_response
            )
            remaining -= 1
